/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   02.03.2015
* @brief  Base interface for SIB access point.
*
* This file is part of the CuteSIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* CuteSIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* CuteSIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Access point implements access method to the SIB (TCP, UDP, Bluetooth)
* and processes requests from agents and sends responses to agents.
* The SIB uses access points as plug-ins.
* This file describe interface for access point that is used by SIB.
**/


#ifndef ACCESSPOINTINTERFACE_H
#define ACCESSPOINTINTERFACE_H

#include "accesspointinterface_global.h"

#include "client.h"
#include "request.h"


// Define indetifier for the access point interface.
#define AccessPointInterface_iid "org.fruct.oss.Projects.SmartM3.AccessPointInterface"

namespace cutesib {
namespace access_point {

//TODO: how to change: dosent store interafce in the request
// Forward declaration to compile (eliminate cross .h includes)
class Request;
class Client;

/**
 * @brief The AccessPointInterface class is used as an interface
 * to represent access poitn for SIB.
 */
class ACCESSPOINTINTERFACESHARED_EXPORT AccessPointInterface
{
public:
    /**
     * @brief Empty virtual destructor for an inheritance.
     */
    virtual ~AccessPointInterface() {}

    /**
     * @brief Sets parameters of the access point.
     * Format of parameter is not defined.
     * Base form is 'key:value;key:value'.
     * @param[in] configId. SIB identifier for access point.
     * @param[in] parameters. Parameters of the access point.
     * @see configId()
     */
    virtual void setParameters(QString configId, QString parameters) = 0;

    /**
     * @brief Getter for SIB config identifier.
     * SIB sets this id and uses it to control access point.
     * This id does not used by access point.
     * @return SIB config identifier.
     * @see setParameters
     */
    virtual QString configId() = 0;

    /**
     * @brief Starts the access point.
     * @return 0 on success or -1 otherwise.
     */
    virtual int start() = 0;

    /**
     * @brief Stops work of access point.
     * The access point stops listening clients, close connection with current clients,
     * remove temorary information about clients connection etc.
     * After calling this method the access point can be started again (#start).
     */
    virtual void stop() = 0;

    /**
     * @brief Sends data to client.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @see Client
     */
    virtual void send(const Client* client, const QByteArray data) = 0;

    //TODO: realise start/stop network chacking

    /**
     * @brief Starts to check connection state of a client (KeepAlive)
     * The client will be checking with some frequency.
     * When the client is no longer available, then the signal
     * (#clientDisconnected()) will be emitted.
     * The method does nothing if client connection state is already checking.
     * @param[in] client. Client to start checking connection.
     * @see clientDisconnected()
     */
    virtual void startChecking(Client* client) = 0;

    /**
     * @brief Stops to check connection state of a client.
     * The method does nothing if client connection state is not checking.
     * @param[in] client. Client to stop checking connection.
     */
    virtual void stopChecking(Client* client) = 0;

    /**
     * @brief Checks client state: connected or not.
     * @param[in] client. Client to check connection.
     * @return true when the client is connected or false otherwise.
     */
    virtual bool isConnected(const Client* client) = 0;

    /**
     * @brief Closes a connection with a client.
     * With closing connection a client will be removed from access point and deleted.
     * The client cannot be used for other action anymore.
     * @param[in] client. Client to close the connection.
     * @see Client
     */
    virtual void close(Client* client) = 0;

protected:
    /**
     * @brief Protected empty constructor to prevent instantiating.
     */
    AccessPointInterface() {}

signals:
    //TODO: add errors codes and code parameter
    /**
     * @brief Signals when the error occurred in the access point.
     * @param data
     */
//    virtual void errorOccured(QString data) = 0;

    //NOTE: save or not client on the access point side
    /**
     * @brief Signals when a client disconnected from the access point.
     * When a client is disconnected (itself or network error),
     * then a client will not be used by the access point.
     * The client instance must be deleted in the slot.
     * @param client that was disconnected.
     */
    virtual void clientDisconnected(Client* client) = 0;

    /**
     * @brief Signals when a new client connected to the access point.
     * @param client. Client that have been connected to access point.
     */
    virtual void clientConnected(cutesib::access_point::Client* client) = 0;

    //TODO: change data to QByteArray
    /**
     * @brief Signals when a client sends a request to access point.
     * @param client. Client that sends a request.
     * @param data. Data that were sended by the client.
     */
    virtual void requestReceived(Client* client, QString data) = 0;
    virtual void requestReceived(Request* request) = 0;
};

} // namespace access_point
} // namespace cutesib

// Declare QT interface
Q_DECLARE_INTERFACE(cutesib::access_point::AccessPointInterface, "AccessPointInterface")

#endif // ACCESSPOINTINTERFACE_H
