/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   08.03.2015
* @brief  Class to manage access points and protocol handlers.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Class loads access points in runtime and links access points with handlers.
**/

#include "accesspointmanager.h"

#include <iostream>

#include <QString>
#include <QDir>
#include <QPluginLoader>
#include <QDebug>

#include "accesspointinterface.h"
#include "ssaphandler.h"
#include "sibdebug.h"
#include "logger.h"

cutesib::AccessPointManager::AccessPointManager(SibConfig *config, QObject *parent) : QObject(parent)
{

    _config = config;


}

cutesib::AccessPointManager::~AccessPointManager()
{

}

void cutesib::AccessPointManager::load(QString accessPointsDir)
{
    //const QDir dir("C:\\Users\\legioner\\GoogleDrive\\Work\\Programming\\smart-m3\\qtsib\\TcpAccessPoint\\debug\\");
    const QDir dir(accessPointsDir);
    //QDir::setCurrent("C:/Users/legioner/GoogleDrive/Work/Programming/smart-m3/qtsib/CuteSIB");

    qDebug() << "Dir" << dir.absoluteFilePath(accessPointsDir);

    foreach (QString fileName, dir.entryList(QDir::Files)) {

        if(!fileName.contains(SIB_AP_EXT)) {
            qDebug() << SIB_AP_EXT << fileName;
            continue;
        }

        QPluginLoader loader(dir.absoluteFilePath(fileName));
        QObject *plugin = loader.instance();

        if (plugin) {
            qDebug() << "Access point file:" << dir.absoluteFilePath(fileName) << endl;

            cutesib::access_point::AccessPointInterface *aPoint = qobject_cast<cutesib::access_point::AccessPointInterface  *>(plugin);

            if (aPoint) {
                QJsonObject json = loader.metaData().value("MetaData").toObject();
                std::cout << "Access point: " << loader.metaData().value("MetaData").toObject().value("name").toString().toStdString()
                          << " version " << json.value("version").toString().toStdString()
                          << " (" << json.value("protocol").toString().toStdString() << ").\n";

                //TODO: Save loader and access point.
                //_ssapHandler_ = new SsapHandler();
                //_ssapHandler_->addAccessPoint(aPoint);
                //aPoint->start();
                //test_ap(a_point, &a);
            }
        } else {
            qWarning() << "Loading access point error:" << loader.errorString() << endl;
        }
    }
}

void cutesib::AccessPointManager::load(SibConfig *config)
{
    foreach(QString id, config->getAccessPoints()) {

        QString fullpath = config->getAccessPointPath(id);

        qDebug() << "Loading" << fullpath;

        QPluginLoader *loader = new QPluginLoader(fullpath);

        if (loader->load()) {

            QJsonObject json = loader->metaData().value("MetaData").toObject();

            Logger::message() << "Loaded access point '" << id
                              << "': "
                              << json.value("name").toString()
                              << " version " << json.value("version").toString()
                              << " ("
                              << json.value("id").toString() << ", "
                              << json.value("protocol").toString() << ")." << endl;

            _loaders.insert(id, loader);
        } else {
            qWarning() << "Loading access point error:" << loader->errorString();
            continue;
        }

  //      QObject *plugin = loader.instance();

//        if (plugin) {
//            cutesib::access_point::AccessPointInterface *accessPoint = qobject_cast<cutesib::access_point::AccessPointInterface*>(plugin);

//            if (accessPoint) {


//                //TODO: Save loader and access point.
//               // ssapHandler_ = new SsapHandler();
//               // ssapHandler_->addAccessPoint(aPoint);
//                //aPoint->start();
//                //test_ap(a_point, &a);
//            }
//        } else {
//            qWarning() << "Loading access point error:" << loader.errorString() << endl;
//        }

    }
}

void cutesib::AccessPointManager::unload(QString accessPointsId)
{
    QPluginLoader *loader = _loaders.take(accessPointsId);

    if (loader) {
        loader->unload();
        delete loader;
    }


}

void cutesib::AccessPointManager::unloadAll()
{
    foreach (QString key, _loaders.keys()) {
        unload(key);
    }
}

void cutesib::AccessPointManager::setAccessPoints(cutesib::SsapHandler *handler)
{
    foreach (QString id, _loaders.keys()) {
        QPluginLoader *loader = _loaders.take(id);
        QString protocol = loader->metaData().value("MetaData").toObject().value("protocol").toString();

        //TODO: Severel handlers support.
        //if (protocol.compare("SSAP/XML", Qt::CaseInsensitive) == 0) {

            QObject *plugin = loader->instance();

            if (plugin) {
               cutesib::access_point::AccessPointInterface *accessPoint = qobject_cast<cutesib::access_point::AccessPointInterface*>(plugin);

               accessPoint->setParameters(id, _config->getAccessPointParameters(id));

               handler->addAccessPoint(accessPoint);
            } else {
                qWarning() << "Can't instance access point:" << id;
            }
        //}
    }
}

