/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   08.03.2015
* @brief  Handler to implement SSAP protocol.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Implementation of SSAP/XML protocol for SIB.
**/

#include "sibinitializer.h"

#include "iostream"

#include <QDir>
#include <QSettings>
#include <QDebug>

#include "sibconfig.h"
#include "siberrors.h"
#include "sibdebug.h"
#include "logger.h"

cutesib::SibInitializer::SibInitializer(QObject *parent) : QObject(parent)
{
    configPath_ = QDir::currentPath() + QDir::separator() + CUTESIB_CONFIG_NAME;

}

cutesib::SibInitializer::SibInitializer(QString configPath, QObject *parent) : QObject(parent)
{
    configPath_ = configPath;
}

cutesib::SibInitializer::~SibInitializer()
{
    delete accessPointManager_;
    delete ssapHandler_;
    delete config_;
}

int cutesib::SibInitializer::initialize()
{
    qDebug() << configPath_;
    // File checking need to do in other place
    QFileInfo checkCurrent(QDir::currentPath() + QDir::separator() + CUTESIB_CONFIG_NAME);
    QFileInfo checkHome(QDir::homePath() + QDir::separator() + ".cutesib" + QDir::separator()  + CUTESIB_CONFIG_NAME);
    QFileInfo setupCheck(configPath_);

    // check if file exists and if yes: Is it really a file and no directory?
    if (setupCheck.exists() && setupCheck.isFile()) {

    } else if (checkCurrent.exists() && checkCurrent.isFile()) {
        qDebug() << "Current path is used for config.";
        configPath_ = checkCurrent.absoluteFilePath();
    } else if (checkHome.exists() && checkHome.isFile()) {
        qDebug() << "Home path is used for config.";
        configPath_ = checkHome.absoluteFilePath();
    } else {
        Logger::message() << "Config file not found."<< endl;
        return -1;
    }

    qDebug() << "Path to config: " << configPath_;
    config_ = new SibConfig(configPath_);


    accessPointManager_ = new AccessPointManager(config_, this);

    accessPointManager_->load(config_);

    ssapHandler_ = new SsapHandler(config_, this);

    scheduler_ = new Scheduler(this);

    connect(ssapHandler_, &SsapHandler::commandReady,
     scheduler_, &Scheduler::commandReceived);


    accessPointManager_->setAccessPoints(ssapHandler_);





    return 0;
}

int cutesib::SibInitializer::start()
{
    int result = ssapHandler_->start();

    if (result != 0) {
        qWarning() << "Starting SSAP handler failed.";
    }

    return (result == 0) ? Error::NONE : Error::FALED;
}

void cutesib::SibInitializer::stop()
{
    ssapHandler_->stop();
}



