/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   08.03.2015
* @brief  Class to initialize and setup modules of the SIB.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Implementation of initialization and setting modules of the SIB.
**/

#ifndef SIBINITIALIZER_H
#define SIBINITIALIZER_H

#include <QObject>
#include <QFileInfo>

#include "sibconfig.h"
#include "accesspointmanager.h"
#include "ssaphandler.h"
#include "scheduler.h"

#define CUTESIB_CONFIG_NAME "cutesib.ini"

namespace cutesib {

class SibInitializer : public QObject
{
    Q_OBJECT
public:
    explicit SibInitializer(QObject *parent = 0);
    explicit SibInitializer(QString configPath, QObject *parent = 0);
    ~SibInitializer();

    /**
     * @brief Initializes the SIB to start.
     * Makes SIB configuration, prepares triple-store, access points and handlers.
     * @return Error::NONE on success or Error::FALED otherwise.
     * @see cutesib::Error
     */
    int initialize();

    /**
     * @brief Starts SIB.
     * Starts all modules of the SIB and access points.
     * Call this method after #initialize().
     * @return Error::NONE on success or Error::FALED otherwise.
     * @see initialize()
     * @see stop()
     * @see cutesib::Error
     */
    int start();

    /**
     * @brief Stops SIB.
     * SIB stops receving any clients recvests, sends unsubscribe messages to clients
     * and disconnects clients from SIB. After that SIB stops all handlers, access points and
     * remove all subscription.
     * The SIB can be started again with #start() method.
     * @see start()
     */
    void stop();


signals:

public slots:

private:

    QString configPath_;    /**< Path to config file, defualt is './'. */
    SibConfig *config_;     /**< SIB configuration. */

    AccessPointManager *accessPointManager_; /**< Manager for access points of the SIB. */

    SsapHandler *ssapHandler_;  /**< Haldler for SSAP protocol and SSAP/XML access points. */

    Scheduler *scheduler_;

};

} // namespace cutesib

#endif // SIBINITIALIZER_H
