/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   02.03.2015
* @brief  Access point that implements TCP protocol.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Access point that implements TCP protocol.
**/

#ifndef TCPACCESSPOINT_H
#define TCPACCESSPOINT_H

#include "discoveryaccesspoint_global.h"

#include <QUdpSocket>
#include <QThread>
#include <QTimer>
#include <QMutex>

#include "accesspointinterface.h"

// Plugin identifier of the access point.
#define DiscoveryAccessPoint_iid "org.fruct.oss.Projects.SmartM3.TcpAccessPoint"
#define DISCOVERY_ACCESS_POINT_PLUGIN_FILE "discovery_access_point.json"

namespace cutesib {
namespace access_point {

/**
 * @brief The TcpAccessPoint class implements access point interface
 * and supports TCP protocol to access SIB.
 * @see AccessPointInterface
 */
class DISCOVERYACCESSPOINTSHARED_EXPORT DiscoveryAccessPoint: public QObject, public AccessPointInterface
{
    Q_OBJECT
    Q_INTERFACES(cutesib::access_point::AccessPointInterface)
    Q_PLUGIN_METADATA(IID DiscoveryAccessPoint_iid FILE DISCOVERY_ACCESS_POINT_PLUGIN_FILE)

public:
    //FIXME: can't connect to port 10010, other ports work
    /**
     * @brief TcpAccessPoint class constructor.
     * @param parent. Parent for the instance.
     * @param port. TCP port to listen, default is 10010.
     */
    explicit DiscoveryAccessPoint(QObject *parent = 0, int port = 10000);

    /**
     * @brief TcpAccessPoint class destructor.
     */
    ~DiscoveryAccessPoint() { qDebug() << "TcpAccessPoint descructor"; }

    /**
     * @brief Sets parameters of the access point.
     * Format of parameters is key-value pairs: 'key:value;key:value;...'.
     * There are tree parameters: 'sib_ip' (IP of the SIB to connect clients,
     * default value is current SIB IP address);
     * 'sib_port' (port of the SIB to connect cilents, default value is 10010);
     * 'port' (port to listen clients requests).
     * @param[in] configId. SIB identifier for access point.
     * @param[in] parameters. SIB parameters for the access point.
     * @see configId()
     */
    void setParameters(QString configId, QString parameters);

    /**
     * @brief Getter for SIB config identifier.
     * SIB sets this id and uses it to control access point.
     * This id does not used by access point.
     * @return SIB config identifier.
     * @see setParameters
     */
    QString configId();

    /**
     * @brief Starts the access point.
     * Starts to listen TCP port for client connection.
     * @return 0 on success or -1 otherwise.
     */
    int start();

    /**
     * @brief Stops work of the access point.
     * Stops to listen TCP port and to check network connections for clients.
     */
    void stop();

    /**
     * @brief Sends data to client.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @see Client
     */
    void send(const Client* client, const QByteArray data);

    /**
     * @brief Checks client state: connected or not.
     * @param[in] client. Client to check connection.
     * @return true when the client is connected or false otherwise.
     */
    bool isConnected(const Client *client);

    /**
     * @brief Starts to check connection state of a client (KeepAlive)
     * The client will be checking with some frequency.
     * When the client is no longer available, then the signal
     * (#clientDisconnected()) will be emitted.
     * The method does nothing if client connection state is already checking.
     * @param[in] client. Client to start checking connection.
     * @see clientDisconnected()
     */
    void startChecking(Client* client);

    /**
     * @brief Stops to check connection state of a client.
     * The method does nothing if client connection state is not checking.
     * @param[in] client. Client to stop checking connection.
     */
    void stopChecking(Client* client);

    /**
     * @brief Closes a connection with a client.
     * With closing connection a client will be removed from access point and deleted.
     * The client cannot be used for other action anymore.
     * @param[in] client. Client to close the connection.
     * @see Client
     */
    void close(Client* client);


signals:

    /**
     * @brief Signal is emitted when a client disconnected from access point.
     * @param client. Disconnected client.
     */
    void clientDisconnected(cutesib::access_point::Client *client);

    /**
     * @brief clientConnected
     * @param client
     */
    void clientConnected(cutesib::access_point::Client *client);
    void requestReceived(cutesib::access_point::Client *client, QString data);
    void requestReceived(cutesib::access_point::Request *request);
    void stopThread();

public slots:
    /**
     * @brief Slot to get data when a client sends a request.
     */
    void readyRead();



private: // Fields
    QUdpSocket socket_;         /**< Server to process UDP connections. */
    int port_;                  /**< Port to listen UDP connections. */
    int sibPort_ = 10010;               /**< Port of the SIB for clients. */
    QString sibIp_;             /**< IP address of the SIB for clients. */

    static const  QByteArray emptyString_; /**< Empty string to check client network state (KeepAlive). */

    QTimer checkingClientTimer_;        /**< Timer to check clients connection state. */
    QMutex checkingClientLock_;         /**< Lock for checking clients. */
    static const int checkingClientTimeout_ = 5000; /**< Interval timeout for timer */

    QString configId_;   /**< Id that is gived by SIB to control access point. */

    static const char parameterValueDelimeter_ = ':' ;  /**< Delimeter for 'parameter:value'. */
    static const char parametersDelimeter_ = ';';       /**< Delimeter for parameters pairs 'parameter:value;parameter:value'. */
    static const QString parameterPort_;       /**< Pramater name of the port parameter (port to listen clients requests). */
    static const QString parameterSibIp_;      /**< Pramater name of the IP address parameter (IP of the SIB for clients). */
    static const QString parameterSibPort_;    /**< Pramater name of the port parameter (port of the SIB for clients). */

    static const QString requestAccessInfoText_;    /**< Request string from client to get IP and port of SIB. */
    static const QString responseIncorrextRequestText_;            /**< String to send error response */


private: // Methods
    /**
     * @brief Sends data to client with given timeout.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @param[in] timeout. Time to wait in milliseconds.
     * @see TcpClient
     */
    //bool send(const TcpClient* client, const QByteArray data, int timeout);

    /**
     * @brief Gets a client by socket.
     * @param[in] socket. Socket of client.
     * @return client with given socket or 0 if the client is new.
     * @see TcpClient
     */
    //TcpClient* getTcpClient(const QTcpSocket *socket);

    bool send(QHostAddress clientAddress, quint16 clientPort, const QString message);

    void setPort(QString parameter);

    void setSibPort(QString parameter);

    void setSibIp(QString parameter);
};

} // namespace access_point
} // namespace cutesib

#endif // TCPACCESSPOINT_H
