/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   02.03.2015
* @brief  Access point that implements TCP protocol.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Access point that implements TCP protocol.
**/

#include <iostream>
#include <QObject>

#include "tcpaccesspoint.h"
#include "tcpclient.h"

#include <QStringList>


const QByteArray cutesib::access_point::TcpAccessPoint::emptyString_ = QByteArray(" ");

cutesib::access_point::TcpAccessPoint::TcpAccessPoint(QObject *parent, int port) : QObject(parent)
{
    port_ = port;

    // Whenever a user connects, it will emit signal.
    connect(&server_, SIGNAL(newConnection()), this, SLOT(newConnection()));

    // Setup timer for checking client connections.
    checkingClientTimer_.setInterval(checkingClientTimeout_);
    connect(&checkingClientTimer_, &QTimer::timeout, this, &TcpAccessPoint::connectionChecking);
}

void cutesib::access_point::TcpAccessPoint::setParameters(QString configId, QString parameters)
{
    configId_ = configId;

    QStringList parametersList = parameters.split(";");

    foreach (QString parameter, parametersList) {
        if (parameter.startsWith("port:")){
            setPortFromParameter(parameter);
        }
    }
}

QString cutesib::access_point::TcpAccessPoint::configId()
{
    return configId_;
}

int cutesib::access_point::TcpAccessPoint::start()
{
    // If the TCP server is already in listening mode then do nothing.
    // If no then start listening on specific port.
    if (server_.isListening()) {
        return -1;
    }

    if(!server_.listen(QHostAddress::Any, port_)) {
        qDebug() << "TCP access point could not start\n";
        return -1;
    }

    qDebug() << "TCP access point started on port" << port_;

    return 0;
}

//TODO: check calling method on stopped server.
void cutesib::access_point::TcpAccessPoint::stop()
{
    qDebug() << SIB_DEBUG_PREFIX << SIB_DEBUG_FNAME << endl;

    // Close all socets and then the TCP server.
    foreach (TcpClient* client, clients_) {
        client->getSocket()->close();
        delete client;
    }

    server_.close();
}

void cutesib::access_point::TcpAccessPoint::send(const Client *client, const QByteArray data)
{
    TcpClient *tcpClient = dynamic_cast<TcpClient*>((Client*)client);

    if (!tcpClient) {
        qWarning() << SIB_DEBUG_PREFIX << SIB_DEBUG_FNAME << "- incorrect client.\n";
        return;
    }

    tcpClient->Lock();
    //BUG: see how it works when the client is disconnected
    send(tcpClient, data, -1); // Waits without timeout
    tcpClient->Unlock();
}

bool cutesib::access_point::TcpAccessPoint::isConnected(const cutesib::access_point::Client *client)
{
    const TcpClient *tcpClient = dynamic_cast<const TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << SIB_DEBUG_PREFIX << SIB_DEBUG_FNAME << "- incorrect client.\n";
        return false;
    }

    tcpClient->getSocket()->write(emptyString_.data(), 1);
    tcpClient->getSocket()->flush();
    tcpClient->getSocket()->waitForBytesWritten(5000);

    return tcpClient->isConnected;
    //TODO: MagicNumber in isConnected
    //return send(tcpClient, emptyString_, 5000);
}

void cutesib::access_point::TcpAccessPoint::startChecking(Client *client)
{
    TcpClient *tcpClient = dynamic_cast<TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << "Incorrect client.";
        return;
    }

    // Block checking clients to add a new one
    // and start timer to interval checking.
    // Timer works only if there are client to check connections.
    checkingClientLock_.lock();

    if(!checkingClients_.contains(tcpClient))
        checkingClients_.insert(tcpClient);

    if (!checkingClientTimer_.isActive())
        checkingClientTimer_.start();

    checkingClientLock_.unlock();
}

void cutesib::access_point::TcpAccessPoint::stopChecking(Client *client)
{
    TcpClient *tcpClient = dynamic_cast<TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << SIB_DEBUG_PREFIX << SIB_DEBUG_FNAME << "- incorrect client.\n";
        return;
    }

    // Block checking clients and remove client from checking list.
    // If there are no clients, then the timer will be stopped.
    checkingClientLock_.lock();

    checkingClients_.remove(tcpClient);

    if (checkingClients_.count() == 0)
        checkingClientTimer_.stop();

    checkingClientLock_.unlock();
}

void cutesib::access_point::TcpAccessPoint::close(Client* client)
{
    TcpClient* tcpClient = dynamic_cast<TcpClient*>(client);

    if (!tcpClient) {
        qWarning() << SIB_DEBUG_PREFIX << SIB_DEBUG_FNAME << "- incorrect client.\n";
        return;
    }

    tcpClient->getSocket()->close();
}

void cutesib::access_point::TcpAccessPoint::newConnection()
{
    // Grab a new socket, create a client based on it
    // and connect apropriate slots to socket's signals.
    QTcpSocket *socket = server_.nextPendingConnection();

    qDebug() << "Client connected:"
             << socket->peerAddress() << socket->peerPort();

    TcpClient* client = new TcpClient(this, socket);
    clients_.insert(client);

    connect(socket, SIGNAL(disconnected()), this, SLOT(socketDisconnected()));
    connect(socket, SIGNAL(readyRead()), this, SLOT(readyRead()));

    emit clientConnected(client);
}

void cutesib::access_point::TcpAccessPoint::readyRead()
{
    QTcpSocket* socket = qobject_cast<QTcpSocket*>(QObject::sender());

    TcpClient *sendClient = getTcpClient(socket);

    sendClient->addMessagePart(socket->readAll());

    qDebug() << "Receved message from:" << socket->peerAddress() << socket->peerPort();

    if (!sendClient->isReceivedFullMessage()) {
        qDebug() << "Messge is not full.";
        return;
    }

    Request *request = new Request(this, sendClient, sendClient->getMessage());

    qDebug() << "Message to SIB:" << request->getData();

    emit requestReceived(new TcpClient(this, socket), QString(receiveBuffer_.data()));
    emit requestReceived(request);
}

void cutesib::access_point::TcpAccessPoint::socketDisconnected()
{
    QTcpSocket* socket = dynamic_cast<QTcpSocket*> (QObject::sender());
    qDebug() << "cleint disconnected:"
             << socket->peerAddress()
             << socket->peerPort()
             << "State: " << getTcpClient(socket)->getSocket()->state();

    getTcpClient(socket)->isConnected = false;

}

bool cutesib::access_point::TcpAccessPoint::send(TcpClient *client, const QByteArray data, int timeout)
{
    qDebug() << "Send bytes to"
             << client->getSocket()->peerAddress()
             << client->getSocket()->peerPort()
             << "\nData:" << data << "\n";

    //    if (client->getSocket()->state() == QAbstractSocket::UnconnectedState) {
    //        ((TcpClient*)client)->isConnected = false;
    //        return false;
    //    }

    // connect(client->getSocket(), SIGNAL(bytesWritten(qint64)), this, SLOT(onWrite(qint64)));

    //if(client->getSocket()->ConnectingState)

//    int i = data.size();
//    QByteArray temp;
//        QDataStream dataBuf(&temp, QIODevice::ReadWrite);
//        data << (int)data.size();

    //client->getSocket()->write() (temp); //write size of data

//    client->setMessage(data);

    //qint64 sendSize = client->getSocket()->write(client->getMessagePart(5000));


client->setMessage(data);
    QByteArray dataPart = client->getMessagePart(1000);

    qDebug()<< "Part " <<dataPart;

    while (!dataPart.isEmpty()) {

            qDebug() << "Send:" << dataPart;
        client->getSocket()->write(dataPart);
        client->getSocket()->flush();
        client->getSocket()->waitForBytesWritten(10000);

        dataPart = client->getMessagePart(1000);

    }

    //qint64 sendSize = client->getSocket()->write(data);//, data.size());

//     connect(client->getSocket(),
//             &QTcpSocket::bytesWritten,
//             this,
//             &TcpAccessPoint::bytesWritten);

    //qDebug() << "Sending size:" << sendSize << data.size();

//    client->getSocket()->flush();
//    client->getSocket()->waitForBytesWritten(10000);

    //disconnect(client->getSocket(), SIGNAL(bytesWritten(qint64)), this, SLOT(onWrite(qint64)));

    return client->isConnected;

    //    if (client->getSocket()->error() == -1) {
    //    //    return true;
    //    }

    //    return retval;
}

cutesib::access_point::TcpClient *cutesib::access_point::TcpAccessPoint::getTcpClient(const QTcpSocket *socket)
{
    foreach (TcpClient *client, clients_) {
        if (client->getSocket() == socket) {
            return client;
        }
    }



    return 0;
}

void cutesib::access_point::TcpAccessPoint::setPortFromParameter(QString parameter)
{
    int port = parameter.remove(0, 5).toInt();

    if (port > 0) {
        port_ = port;
    }
}

void cutesib::access_point::TcpAccessPoint::connectionChecking()
{
    int i = 0;

    qDebug() << "Start connection checking.";

    // Check connections.
    // If the connections with client is lost,
    // then the signal is emitted and client will be removed from checking.
    // Cycle works while there are client for checking.
    while(true) {
        checkingClientLock_.lock();

        if (checkingClients_.empty()) {
            checkingClientTimer_.stop();
            checkingClientLock_.unlock();
            break;
        } else if (i >= checkingClients_.count()){
            checkingClientLock_.unlock();
            break;
        }

        TcpClient *client = checkingClients_.values().at(i);

        qDebug() << "Checking for"
                 << client->getSocket()->peerAddress()
                 << client->getSocket()->peerPort();

        client->Lock();
        bool connected = isConnected(client);
        client->Unlock();
        if (!connected) {
            qDebug() << "Connection is lost with"
                     << client->getSocket()->peerAddress()
                     << client->getSocket()->peerPort();
            checkingClients_.remove(client);
            clients_.remove(client);
            emit clientDisconnected(client);
        }

        checkingClientLock_.unlock();

        ++i;
    }

    qDebug() << "End connection checking. Checked '" << i << "' clients.";
}

void cutesib::access_point::TcpAccessPoint::bytesWritten(qint64 bytesWritten)
{
    qDebug() << bytesWritten;

    return;
    QTcpSocket* socket = qobject_cast<QTcpSocket*>(QObject::sender());

    TcpClient *sendClient = getTcpClient(socket);

    QByteArray data = sendClient->getMessagePart(1000);

    if (data.isEmpty()) {
        disconnect(sendClient->getSocket(),
                   &QTcpSocket::bytesWritten,
                   this,
                   &TcpAccessPoint::bytesWritten);
    } else {
        qDebug() << "Send:" << data;
        sendClient->getSocket()->write(data);
    }
}


