/**
* @author Aleksandr A. Lomov <lomov@cs.karelia.ru>
* @date   02.03.2015
* @brief  Access point that implements TCP protocol.
*
* This file is part of the QT-SIB project.
*
* Copyright (C) 2015 by PetrSU.
* Contact: http://oss.fruct.org/wiki/SmartSpace-platform
*
* @section LICENSE
*
* QT-SIB is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* QT-SIB is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with QT-SIB; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor,
* Boston, MA  02110-1301  USA
*
* @section DESCRIPTION
*
* Access point that implements TCP protocol.
**/

#ifndef TCPACCESSPOINT_H
#define TCPACCESSPOINT_H

#include "tcpaccesspoint_global.h"

#include <QTcpSocket>
#include <QTcpServer>
#include <QThread>
#include <QTimer>
#include <QMutex>

#include "accesspointinterface.h"
#include "tcpclient.h"

// Plugin identifier of the access point.
#define TcpAccessPoint_iid "org.fruct.oss.Projects.SmartM3.TcpAccessPoint"
#define TCP_ACCESS_POINT_PLUGIN_FILE "tcp_access_point.json"

namespace cutesib {
namespace access_point {

/**
 * @brief The TcpAccessPoint class implements access point interface
 * and supports TCP protocol to access SIB.
 * @see AccessPointInterface
 */
class TCPACCESSPOINTSHARED_EXPORT TcpAccessPoint: public QObject, public AccessPointInterface
{
    Q_OBJECT
    Q_INTERFACES(cutesib::access_point::AccessPointInterface)
    Q_PLUGIN_METADATA(IID TcpAccessPoint_iid FILE TCP_ACCESS_POINT_PLUGIN_FILE)

public:
    //FIXME: can't connect to port 10010 on Windows, other ports work
    /**
     * @brief TcpAccessPoint class constructor.
     * @param parent. Parent for the instance.
     * @param port. TCP port to listen, default is 10010.
     */
    explicit TcpAccessPoint(QObject *parent = 0, int port = 10010);

    /**
     * @brief TcpAccessPoint class destructor.
     */
    ~TcpAccessPoint() { qDebug() << "TcpAccessPoint descructor"; }

    /**
     * @brief Sets parameters of the access point.
     * Format of parameters is key-value pairs: 'key:value;key:value;...'.
     * Now there is one parameter: port (default value is 10010).
     * @param[in] configId. SIB identifier for access point.
     * @param[in] parameters. SIB parameters for the access point.
     * @see configId()
     */
    void setParameters(QString configId, QString parameters);

    /**
     * @brief Getter for SIB config identifier.
     * SIB sets this id and uses it to control access point.
     * This id does not used by access point.
     * @return SIB config identifier.
     * @see setParameters
     */
    QString configId();

    /**
     * @brief Starts the access point.
     * Starts to listen TCP port for client connection.
     * @return 0 on success or -1 otherwise.
     */
    int start();

    /**
     * @brief Stops work of the access point.
     * Stops to listen TCP port and to check network connections for clients.
     */
    void stop();

    /**
     * @brief Sends data to client.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @see Client
     */
    void send(const Client* client, const QByteArray data);

    /**
     * @brief Checks client state: connected or not.
     * @param[in] client. Client to check connection.
     * @return true when the client is connected or false otherwise.
     */
    bool isConnected(const Client *client);

    /**
     * @brief Starts to check connection state of a client (KeepAlive)
     * The client will be checking with some frequency.
     * When the client is no longer available, then the signal
     * (#clientDisconnected()) will be emitted.
     * The method does nothing if client connection state is already checking.
     * @param[in] client. Client to start checking connection.
     * @see clientDisconnected()
     */
    void startChecking(Client* client);

    /**
     * @brief Stops to check connection state of a client.
     * The method does nothing if client connection state is not checking.
     * @param[in] client. Client to stop checking connection.
     */
    void stopChecking(Client* client);

    /**
     * @brief Closes a connection with a client.
     * With closing connection a client will be removed from access point and deleted.
     * The client cannot be used for other action anymore.
     * @param[in] client. Client to close the connection.
     * @see Client
     */
    void close(Client* client);

signals:

    /**
     * @brief Signal is emitted when a client disconnected from access point.
     * @param client. Disconnected client.
     */
    void clientDisconnected(cutesib::access_point::Client *client);

    /**
     * @brief clientConnected
     * @param client
     */
    void clientConnected(cutesib::access_point::Client *client);
    void requestReceived(cutesib::access_point::Client *client, QString data);
    void requestReceived(cutesib::access_point::Request *request);
    void stopThread();

public slots:
    /**
     * @brief Slot to process a new TCP connections.
     */
    void newConnection();

    /**
     * @brief Slot to get data when a client sends a request.
     */
    void readyRead();

    /**
     * @brief Slot to process client disconnection from the access point.
     */
    void socketDisconnected();


private: // Fields
    QTcpServer server_;         /**< Server to process TCP connections. */
    QSet<TcpClient*> clients_;  /**< TCP clients. */
    int port_;                  /**< Port to listen TCP connections. */

    const static QByteArray emptyString_; /**< Empty string to check client network state (KeepAlive). */

    QTimer checkingClientTimer_;        /**< Timer to check clients connection state. */
    QMutex checkingClientLock_;         /**< Lock for checking clients. */
    QSet<TcpClient*> checkingClients_;  /**< TCP clients to check connections. */
    static const int checkingClientTimeout_ = 5000; /**< Interval timeout for timer */

    QString configId_;   /**< Id that is gived by SIB to control access point. */

    QByteArray receiveBuffer_;

private: // Methods
    /**
     * @brief Sends data to client with given timeout.
     * @param[in] client. Client to send the data.
     * @param[in] data. Data to send.
     * @param[in] timeout. Time to wait in milliseconds.
     * @see TcpClient
     */
    bool send(TcpClient *client, const QByteArray data, int timeout);

    /**
     * @brief Gets a client by socket.
     * @param[in] socket. Socket of client.
     * @return client with given socket or 0 if the client is new.
     * @see TcpClient
     */
    TcpClient* getTcpClient(const QTcpSocket *socket);

    void setPortFromParameter(QString parameter);



private slots:
    /**
     * @brief Slot checks connections state for clients.
     * This slot is used with a timer.
     * @see checkingClientTimer_
     */
    void connectionChecking();

    void bytesWritten(qint64 bytesWritten);

};

} // namespace access_point
} // namespace cutesib

#endif // TCPACCESSPOINT_H
